package com.qubole.spark.hiveacid.reader.v2

import com.qubole.spark.hiveacid.rdd.HiveAcidPartition
import com.qubole.spark.hiveacid.util.{HiveAcidCommon, SerializableConfiguration, SerializableWritable}
import org.apache.spark.broadcast.Broadcast
import org.apache.spark.sql.sources.v2.reader.InputPartition
import org.apache.spark.sql.vectorized.ColumnarBatch
import org.apache.hadoop.io.Writable
import org.apache.hadoop.mapred.InputFormat
import org.apache.hadoop.mapred.JobConf
import org.apache.spark.sql.sources.v2.reader.InputPartitionReader
import org.apache.spark.sql.catalyst.InternalRow
import org.apache.spark.sql.types._
import org.apache.spark.internal.Logging

private[hiveacid] class HiveAcidInputPartitionV2(split: HiveAcidPartition,
                                                 var conf: Broadcast[SerializableConfiguration],
                                                 initLocalJobConfFuncOpt: Option[JobConf => Unit],
                                                 splitPath : String,
                                                 partitionValues : InternalRow,
                                                 requiredFields: Array[StructField],
                                                 partitionSchema : StructType,
                                                 isFullAcidTable: Boolean)
                  extends InputPartition[ColumnarBatch] with Logging {
  override def preferredLocations: Array[String] = {
    try split.inputSplit.value.getLocations
    catch {
      case e: Exception =>
        //preferredLocations specifies to return empty array if no preference
        new Array[String] (0)
    }
  }

  def setBroadcastConfig (broadcastCfg: Broadcast[SerializableConfiguration]): Unit = {
    conf = broadcastCfg
  }

  override def createPartitionReader: InputPartitionReader[ColumnarBatch] = {
    val jobConf = if (initLocalJobConfFuncOpt != None)
      HiveAcidCommon.getJobConf(conf.value.value,
        splitPath, true, initLocalJobConfFuncOpt)
      else
        new JobConf (conf.value.value)

    new HiveAcidInputPartitionReaderV2(split, jobConf, partitionValues,
      requiredFields, partitionSchema, isFullAcidTable)
  }
}
